import click

from src.core import NoteMe


OPEN_BROWSER = 'open_browser'
REMOVE = 'remove'


def helper_for_action(note_me: NoteMe, link_chosen: str, action: str) -> None:
    if action.lower() == OPEN_BROWSER:
        click.launch(link_chosen)
    elif action.lower() == REMOVE:
        note_me.remove_link(link_chosen)


def helper_for_hostname_action(note_me: NoteMe, link_chosen: str, action: str) -> None:
    if action.lower() == OPEN_BROWSER:
        click.launch('https://' + link_chosen)
    elif action.lower() == REMOVE:
        note_me.remove_hostname(link_chosen)



from typing import Union, List

import click

from src.promps.list import (
    ask_to_choose_hostname,
    ask_to_choose_action,
    ask_to_choose_hostname_with_action,
)
from src.promps.search import ask_for_search_action
from src.core import NoteMe
from src.core import helpers


@click.group()
@click.pass_context
def cli(ctx):
    ctx.obj = NoteMe()


@cli.command()
@click.argument('link', nargs=-1)
@click.pass_obj
def nsave(note_me: NoteMe, link: Union[str, List[str]]) -> None:
    note_me.bulk_save(link)


@cli.command()
@click.option('--list-host', is_flag=True)
@click.option('--reverse/--no-reverse', default=False)
@click.option('-h', '--hostname', type=str)
@click.pass_obj
def nlist(note_me: NoteMe, list_host: bool, reverse: bool, hostname: str) -> None:
    if list_host:
        link_chosen, action = ask_to_choose_hostname_with_action(note_me, reverse)
        if not link_chosen:
            click.echo(f'hostname is empty')
            return

        helpers.helper_for_hostname_action(note_me=note_me, link_chosen=link_chosen, action=action)

    else:
        hostname_ = ask_to_choose_hostname(note_me, reverse) if not hostname else hostname

        if note_me.is_empty_list_for(hostname_):
            if hostname_:
                click.echo(f'link for hostname "{hostname_}" is empty')
            else:
                click.echo(f'hostname is empty')
            return

        link_chosen, action = ask_to_choose_action(note_me, hostname_)
        helpers.helper_for_action(note_me=note_me, link_chosen=link_chosen, action=action)


@cli.command()
@click.argument('search', type=str)
@click.option('-l', '--limit', type=int)
@click.option('-h', '--hostname', type=str)
@click.pass_obj
def nsearch(note_me: NoteMe, search: str, limit: int, hostname: str) -> None:
    list_links = note_me.search(search_value=search, limit=limit, hostname=hostname)

    if len(list_links) == 0:
        click.echo('Your search did found')
        return

    link_chosen, action = ask_for_search_action(list_links)
    helpers.helper_for_action(note_me=note_me, link_chosen=link_chosen, action=action)



import inquirer

from src.core import NoteMe


def ask_to_choose_hostname(note_me, reverse=None):
    list_sites = note_me.list_hostname()

    if reverse:
        list_sites.sort(reverse=reverse)

    questions = [
        inquirer.List(
            name='hostname',
            message='Choose list from hostname',
            choices=list_sites,
        )
    ]

    answer = inquirer.prompt(questions).get('hostname') if len(list_sites) > 0 else None
    return answer


def ask_to_choose_action(note_me: NoteMe, hostname, reverse=None):
    list_links = note_me.get_links(hostname)

    if reverse:
        list_links.sort(reverse=reverse)

    questions = [
        inquirer.List(
            name='link_chosen',
            message='Choose link',
            choices=list_links,
        ),
        inquirer.List(
            name='action',
            message='What action do you want',
            choices=['open_browser', 'remove']
        )
    ]
    answers = inquirer.prompt(questions)
    return answers['link_chosen'], answers['action']


def ask_to_choose_hostname_with_action(note_me: NoteMe, reverse=None):
    list_hostname = note_me.list_hostname()

    if reverse:
        list_hostname.sort(reverse=reverse)

    if len(list_hostname) == 0:
        return None, None

    questions = [
        inquirer.List(
            name='hostname',
            message='Choose Host',
            choices=list_hostname,
        ),
        inquirer.List(
            name='action',
            message='What action do you want',
            choices=['open_browser', 'remove']
        )
    ]
    answers = inquirer.prompt(questions)
    return answers['hostname'], answers['action']



import os
from configobj import ConfigObj

BASE_DIR = os.path.dirname(os.path.dirname(__file__))


config = ConfigObj(
    os.path.join(os.path.dirname(BASE_DIR), 'notelink_cli', 'src', 'settings.conf')
)



from typing import Union, List
from urllib.parse import urlparse
from src.configs import config as main_config


URL_SAVED_NAME = 'URL_SAVED'


def get_hostname_from_url(url):
    return urlparse(url).hostname


def ensure_config_hostname(config, hostname):
    """
    Ensure doesnt make any error
    when hostname configure want to be filled
    """
    if hostname and hostname not in config[URL_SAVED_NAME].keys():
        config[URL_SAVED_NAME][hostname] = []


class NoteMe:
    def __init__(self, config=None):
        self.config = config or main_config

    def save(self, link):
        hostname = get_hostname_from_url(link)
        ensure_config_hostname(self.config, hostname=hostname)

        list_urls = self.config[URL_SAVED_NAME][hostname]
        list_urls.append(link) if link not in list_urls else None
        self.config.write()

    def bulk_save(self, links: Union[str, List[str]]):
        for link in links:
            self.save(link)

    def search(self, search_value, limit=None, hostname=None):
        search_hostnames = [
            h for h in self.config['URL_SAVED'] if h == hostname
        ] if hostname else self.config['URL_SAVED']

        list_links = []
        for hostname in search_hostnames:
            for link in self.config['URL_SAVED'][hostname]:
                if search_value in link:
                    list_links.append(link)

        return list_links[:limit] if isinstance(limit, int) else list_links

    def list_hostname(self):
        return self.config['URL_SAVED'].keys()

    def get_links(self, hostname):
        link = []
        if hostname in self.config['URL_SAVED'].keys():
            link = self.config['URL_SAVED'][hostname]
        return link

    def is_empty_list_for(self, hostname):
        ensure_config_hostname(config=self.config, hostname=hostname)
        return len(self.config['URL_SAVED'][hostname]) < 1 if hostname else True

    def remove_link(self, link):
        hostname = get_hostname_from_url(link)
        ensure_config_hostname(config=self.config, hostname=hostname)
        self.config['URL_SAVED'][hostname].remove(link)
        self.config.write()

    def remove_hostname(self, hostname):
        del self.config['URL_SAVED'][hostname]
        self.config.write()



import inquirer


def ask_for_search_action(list_links=None):
    list_links = list_links or []

    questions = [
        inquirer.List(
            name='link_chosen',
            message='Choose link',
            choices=list_links,
        ),
        inquirer.List(
            name='action',
            message='What action do you want',
            choices=['open_browser', 'remove']
        )
    ]

    answers = inquirer.prompt(questions)
    return answers['link_chosen'], answers['action']



from setuptools import setup, find_packages

setup(
    name='notelink',
    version='0.0.1',
    description='Management your favorite link using CLI.',
    long_description='Management your favorite link using CLI.',
    author='Agung Yuliyanto',
    author_email='agung.96tm@gmail.com',
    url='https://github.com/agung96tm/notelink',
    project_urls={
        'Source Code': 'https://github.com/agung96tm/notelink',
    },
    license_files=('LICENCE',),
    packages=find_packages(
        exclude=['tests', 'docs']
    ),
    include_package_data=True,
    install_requires=[i.strip() for i in open("requirements.txt").readlines()],
    entry_points={
        'console_scripts': [
            'notelink = src.main:cli',
        ],
    },
    classifiers=[
        "Intended Audience :: Education",
        "Intended Audience :: Developers",
        "Programming Language :: Python :: 3",
        "Operating System :: OS Independent",
    ]
)
